/************************************************************************
 *   IRC - Internet Relay Chat, doc/example_module.c
 *   Copyright (C) 2001 Hybrid Development Team
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 1, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *   $Id: m_svsnick.c,v 1.1 2004/01/11 16:28:13 stacy Exp $
 */

/* List of ircd includes from ../include/ */
#include "stdinc.h"
#include "handlers.h"
#include "client.h"
#include "common.h"     /* FALSE bleah */
#include "ircd.h"
#include "irc_string.h"
#include "numeric.h"
#include "fdlist.h"
#include "s_bsd.h"
#include "s_conf.h"
#include "s_log.h"
#include "s_serv.h"
#include "send.h"
#include "msg.h"
#include "parse.h"
#include "modules.h"

#include "s_user.h"  /* send_umode_out() */
#include "channel_mode.h" /* MODE_ADD & MOD_DEL */
#include "hash.h"


/* Declare the void's initially up here, as modules dont have an
 * include file, we will normally have client_p, source_p, parc
 * and parv[] where:
 *
 * client_p == client issuing command
 * source_p == where the command came from
 * parc     == the number of parameters
 * parv     == an array of the parameters
 */
static void ms_svsnick(struct Client *client_p, struct Client *source_p,
		    int parc, char *parv[]);
static void mo_svsnick(struct Client *client_p, struct Client *source_p,
                    int parc, char *parv[]);

static int clean_nick_name(char *nick);

/* CHGNICK/SVSNICK/SETNICK */
struct Message svsnick_msgtab = {
  "SVSNICK", 0, 0, 3, 3, MFLG_SLOW, 0,
  {m_ignore, m_ignore, ms_svsnick, m_ignore}
};

#ifndef STATIC_MODULES
/* Here we tell it what to do when the module is loaded */
void
_modinit(void)
{
  mod_add_cmd(&svsnick_msgtab);
}

/* here we tell it what to do when the module is unloaded */
void
_moddeinit(void)
{
  mod_del_cmd(&svsnick_msgtab);
}

/* When we last modified the file (shown in /modlist), this is usually:
 */
const char *_version = "$Revision: 1.1 $";
#endif

/*
 * mo_svsnick
 *    parv[0] = sender prefix
 *    parv[1] = target nickname
 *    parv[2] = new nickname
 */
static void mo_svsnick(struct Client *client_p, struct Client *source_p,
                   int parc, char *parv[])
{
  struct Client *target_p;
  struct Client *myuser_p;
  char *nick, newnick[NICKLEN+1];

  nick = parv[1];
  strcpy(newnick, parv[2]);

  if ((target_p = find_client(nick)) != NULL)
    {
      if (!clean_nick_name(newnick))
      {
        sendto_one(source_p, form_str(ERR_ERRONEUSNICKNAME), me.name, source_p->name, newnick);
        return;
      }
  
      if (find_nick_resv(newnick))
      {
        sendto_one(source_p, form_str(ERR_UNAVAILRESOURCE), me.name, source_p->name, newnick);
        return;
      }

      if ((myuser_p = find_client(newnick)))
      {
 
        if (myuser_p == target_p)
        {
          if (strcmp(target_p->name, newnick))
          {
            if (MyClient(target_p))
              change_local_nick(target_p, target_p, newnick);
            else
              sendto_server(client_p, NULL, NOCAPS, NOCAPS, ":%s SVSNICK %s %s", parv[0], target_p->name, newnick);
          }
          else
            return;
        }
  
        else 
        {
          sendto_one(source_p, form_str(ERR_NICKNAMEINUSE), me.name, source_p->name, newnick);
          return;
        }  
      }

      else
      {
        if (MyClient(target_p))
          change_local_nick(target_p, target_p, newnick);
        else
          sendto_server(client_p, NULL, NOCAPS, NOCAPS, ":%s SVSNICK %s %s", parv[0], target_p->name, newnick);
      }
    }
  else
    {
      sendto_one(source_p, form_str(ERR_NOSUCHNICK), me.name, parv[0], nick);
      return;
    }

  /*sendto_realops_flags(UMODE_ALL, L_ALL, "%s Used CHGNICK On %s", source_p->name, nick);*/ /* This doesnt need to be noisey. -jts */
  ilog(L_NOTICE,"%s Used SVSNICK On %s To Change Their Nick To %s", get_oper_name(source_p), nick, newnick);

  return; 
} 

static void ms_svsnick(struct Client *client_p, struct Client *source_p,
			int parc, char *parv[])
{
  struct Client *target_p;
  char *nick, newnick[NICKLEN+1];

  if (parc != 3)
    return;

  nick = parv[1];
  strcpy(newnick, parv[2]);

  target_p = find_client(nick);

  if (!MyClient(target_p))
    sendto_server(client_p, NULL, NOCAPS, NOCAPS, ":%s SVSNICK %s %s", parv[0], target_p->name, newnick);
  else
    change_local_nick(target_p, target_p, newnick);

  /*sendto_realops_flags(UMODE_ALL, L_ALL, "%s Used CHGNICK On %s", parv[0], nick);*/ /* This doesnt need to be noisey. -jts */
  ilog(L_NOTICE, "%s Used SVSNICK On %s To Change Their Nick To %s", parv[0], nick, newnick);

  return;
}

/* clean_nick_name()
 *
 * input        - nickname
 * output       - none
 * side effects - walks through the nickname, returning 0 if erroneous
 */
static int clean_nick_name(char *nick)
{
  assert(nick);
  if(nick == NULL)
    return 0;

  /* nicks cant start with a digit or -, and must have a length */
  if (*nick == '-' || IsDigit(*nick) || !*nick)
    return 0;

  for(; *nick; nick++)
  {
    if(!IsNickChar(*nick))
      return 0;
  }
  
  return 1;
}

/* END OF EXAMPLE MODULE */

